/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.junit.tests.nScopingWhile;

import org.jabhts.annotation.element.EHtml;
import org.jabhts.annotation.element.EText;
import org.jabhts.annotation.flowControl.EScopeChilds;
import org.jabhts.annotation.flowControl.EScopingWhile;
import org.jabhts.annotation.flowControl.EWhile;
import org.jabhts.annotation.global.TCompile;
import org.jabhts.annotation.global.TOverrides;
import org.jabhts.junit.lib.TestBase;

@TCompile
public class TestScopingWhile extends TestBase
{
	@EText
	String getTextOne()
	{
		return "Making a test";
	}

	@EText
	String getString1()
	{
		return "Outer-string-1";
	}

	@EHtml
	void printString2()
	{
		pw.htmlEncode("Outer-string-2");
	}

	//-----------------------------------------------

	int blockOneCount;
	String blockOneName;

	@EWhile
	boolean testBlockOne()
	{
		blockOneCount++;
		blockOneName = "scope-1-get #" + blockOneCount;
		return blockOneCount <= 5;
	}

	@EScopeChilds
	class BlockOne
	{
		@EText
		@TOverrides
		String getString1()
		{
			return blockOneName;
		}

		@EHtml
		@TOverrides
		void printString2()
		{
			pw.htmlEncode("scope-1-print-" + blockOneCount);
		}
	}

	//-----------------------------------------------

	int blockTwoCount;

	@EScopingWhile
	boolean testBlockTwo()
	{
		blockTwoCount++;
		return blockTwoCount <= 3;
	}

	BlockTwo newBlockTwo()
	{
		return new BlockTwo("scope-2-get #" + blockTwoCount);
	}

	class BlockTwo
	{
		final String msg;

		BlockTwo() throws Exception
		{
			throw new Exception("Should not be called");
		}

		BlockTwo(String msg1)
		{
			this.msg = msg1;
		}

		@EText
		@TOverrides
		String getString1()
		{
			return msg;
		}

		@EHtml
		@TOverrides
		void printString2()
		{
			pw.htmlEncode("scope-2-print-" + blockTwoCount);
		}
	}

	//-----------------------------------------------

	int blockThreeCount;

	@EWhile
	boolean testBlockThree()
	{
		blockThreeCount++;
		return blockThreeCount <= 2;
	}

	BlockThree newBlockThree()
	{
		return new BlockThree("scope-3-get #" + blockThreeCount);
	}

	@EScopeChilds
	class BlockThree
	{
		final String msg;

		BlockThree() throws Exception
		{
			throw new Exception("Should not be called");
		}

		BlockThree(String msg1)
		{
			this.msg = msg1;
		}

		@EText
		@TOverrides
		String getString1()
		{
			return msg;
		}

		@EHtml
		@TOverrides
		void printString2()
		{
			pw.htmlEncode("scope-3-print-" + blockThreeCount);
		}
	}

	//-----------------------------------------------

	@EHtml(tag = "title")
	void printTitle()
	{
		pw.htmlEncode("Testing gone amuck in iterating");
	}

	protected void emitTemplate() throws Exception
	{
		new TestScopingWhileTp(this).emit(pw);
	}
}
