/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.attribute.sub;

import org.jabhts.translator.compiler.application.baseclass.AbstractBooleanApplication;
import org.jabhts.translator.compiler.codeGeneration.api.PatchedGoto;
import org.jabhts.translator.compiler.tree.ElementEmitters;
import org.jabhts.translator.compiler.utils.CompilerException;

class StyleDisplayEmitter extends SubAttributeEmitter<AbstractBooleanApplication>
{
	final boolean inverse;
	final String visibleValue;

	protected StyleDisplayEmitter(ElementEmitters emitters,
	                              AbstractBooleanApplication handler,
	                              boolean inverse, String visibleValue)
	{
		super(emitters, handler, "style", "display");
		this.inverse = inverse;
		this.visibleValue = visibleValue;
	}

	public void generateSubAttribute(String itemPrefix) throws CompilerException
	{
		program.addEmitString(itemPrefix);
		program.addEmitString("display: ");
		handler.emitBooleanCallback(program);
		if (inverse)
			program.addNegateBoolean();
		PatchedGoto handleTrue = program.addPatchedConditionalGoto();
		program.addEmitString("none");
		PatchedGoto done = program.addPatchedGoto();
		handleTrue.patchToCurrentAddress();
		program.addEmitString(visibleValue);
		done.patchToCurrentAddress();
	}
}