/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.baseclass;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Iterator;
import org.jabhts.translator.compiler.application.scope.TargetingInfo;
import org.jabhts.translator.compiler.codeGeneration.api.IteratorInfo;
import org.jabhts.translator.compiler.codeGeneration.api.Program;
import org.jabhts.translator.compiler.utils.CompilerException;

public abstract class AbstractIteratorApplication extends AbstractValueApplication
{
	public final Type itemType;

	protected AbstractIteratorApplication(TargetingInfo target) throws CompilerException
	{
		super(target, false, Iterable.class, Iterator.class);
		Type t = (method != null) ? method.getGenericReturnType() : field.getGenericType();
		if (!(t instanceof ParameterizedType))
			throw makeInvalidAnnotationApplianceException("Iterator/able must be generic parameterized type.");
		ParameterizedType pt = (ParameterizedType)t;
		Type[] ata = pt.getActualTypeArguments();
		if (ata == null || ata.length != 1)
			throw makeInvalidAnnotationApplianceException("Iterator/able must have exactly 1 generic parameterized type argument.");
		itemType = ata[0];
	}

	public IteratorInfo makeIteratorInfo(Program program) throws CompilerException
	{
		String prefix = program.getNextVariableNamePrefix();
		boolean isIterable = Iterable.class.isAssignableFrom(resultType);
		return new IteratorInfo(scope, target.annotationToString(), isIterable, prefix, itemType);
	}

	public void emitIteratorCallback(Program program, IteratorInfo iterInfo) throws CompilerException
	{
		switch (callbackType) {
			case Field:
				program.addGetIteratorField(scope, field, iterInfo);
				break;
			case Method:
				program.addCallback(iterInfo, method);
				break;
			default:
				throw new CompilerException("Unsupported callbackType = " + callbackType + ": " + annotationToString());
		}
	}
}
