/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
	This file implements the setuid-root helper tool.
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/sysctl.h>
#include <SystemConfiguration/SCDynamicStoreCopySpecific.h>
#include "GBDebug.h"
#import "NetworkSettingsUpdater.h"

@interface NetworkSettingsTool : NSObject {
@private
}

- (instancetype)init;

@end

@implementation NetworkSettingsTool

- (instancetype)init
{
	if (!(self = [super init]))
		return nil;
	return self;
}


- (void)exitWithError:(NSString*)msg
{
	NSLog(@"NetworkSettingsTool failed: %@", msg);
	exit(1);
}

- (void)runWithArgs:(NSArray*)args
{
	if (geteuid()) {
		[self exitWithError:[NSString stringWithFormat:@"Not running as root but with uid = %d and euid = %d",
							 getuid(), geteuid()]];
	}
	if (getuid()) {
		if (setuid(0) < 0)
			NSLog(@"Failed setting uid to 0");
		if (getuid()) {
			[self exitWithError:[NSString stringWithFormat:@"Failed to set uid to 0, is still %d, euid = %d",
								 getuid(), geteuid()]];
		}
	}
	if ([args count] < 2)
		[self exitWithError:@"No arguments given"];
	int port = [args[1] intValue];
	NSLog(@"port = %d", port);
	NetworkSettingsUpdater* updater = [[NetworkSettingsUpdater alloc]
										initWithProxyPort:port
										withWantEnable:YES];
	[updater updateSettingsWithCommit:YES];
	NSLog(@"Network settings synchronized.");
	exit(0);
}

@end

int main (int argc, const char * argv[])
{
	@autoreleasepool {
		NSArray *args = [[NSProcessInfo processInfo] arguments];
		NetworkSettingsTool* tool = [[NetworkSettingsTool alloc] init];
		@try {
			[tool runWithArgs:args];
		}
		@catch (NSException* ex) {
			NSLog(@"Stops due to exception: '%@' / '%@'", [ex name], [ex reason]);
			exit(1);
		}
	}
	return 0;
}
