/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//#define DEBUG 1

#import <ExceptionHandling/NSExceptionHandler.h>
#import <SecurityInterface/SFAuthorizationView.h>
#import "GlimmerBlockerPane.h"
#import "NetworkSettingsUpdaterUI.h"
#import "GBDebug.h"
#import "ServerCommunication.h"
#import "FilterTabController.h"
#import "HistoryController.h"
#import "SuspectsController.h"
#import "AlertHelper.h"
#import "Prefs.h"
#import "XmlDoc.h"
#import "Filter.h"
#import "NamedEnum.h"
#import "InstallationHelper.h"
#import "AutoZap.h"

// man malloc_history

#ifdef DEBUG
//#define DEBUG_FSEVENTS
#endif

@interface GlimmerBlockerPrefsPane ()

@property Prefs* prefs;
@property AlertHelper* alertHelper;
@property ServerCommunication* comm;
@property FilterTabController* filterTabController;
@property HistoryController* history;
@property SuspectsController* suspects;
@property InstallationHelper* installationHelper;
@property BOOL hasUpdatedAuthRightsDefinition;
@property BOOL hasAddedWindowNotifications;
@property NSString* glimmerBlockerVersionText;
@property FSEventStreamRef fsEventStreamRef;

@property (weak) IBOutlet NSView*				mainPrefsPaneView;
@property (weak) IBOutlet NSTabView*			tabView;
@property (weak) IBOutlet SFAuthorizationView*	authView;
@property (weak) IBOutlet NSTextField*			footerUrlTextField;
//
@property (weak) IBOutlet NSButton*				setupEnableGlimmerBlockerCB;
//
@property (weak) IBOutlet NSButton*				setupUpdatesAutoCB;
@property (weak) IBOutlet NSButton*				setupUpdatesNowBtn;
@property (weak) IBOutlet NSButton*				setupUpdatesNowBtn2;
@property (weak) IBOutlet NSTextField*			setupUpdatesLastDateTextField;
//
@property IBOutlet NSTextView*					setupCopyrightTextView;
//
@property (weak) IBOutlet NSTabViewItem*		filtersTabViewItem;
@property (weak) IBOutlet NSView*				filtersContainerView;
@property (weak) IBOutlet NSTextField*			filtersMissingServerLabel;
//
@property (weak) IBOutlet NSView*				historyContainerView;
@property (weak) IBOutlet NSTextField*			historyMissingServer;
//
@property (weak) IBOutlet NSView*				suspectsContainerView;
@property (weak) IBOutlet NSTextField*			suspectsMissingServer;
//
@property (weak) IBOutlet NSTextField*			networkPortNumberTextField;
@property (weak) IBOutlet NSButton*				networkAllowOtherComputersCB;
@property (weak) IBOutlet NSButton*				networkSyncNetworkSettingsNowBtn;
@property (weak) IBOutlet NSButton*				networkConfirmNetworkSettingsCB;
@property (weak) IBOutlet NSMatrix*				networkSiteProxyMatrix;
@property (weak) IBOutlet NSButtonCell*			networkSiteProxyEnabledBtn;
@property (weak) IBOutlet NSTextField*			networkSiteProxyHostLabel;
@property (weak) IBOutlet NSTextField*			networkSiteProxyHostTextField;
@property (weak) IBOutlet NSTextField*			networkSiteProxyPortLabel;
@property (weak) IBOutlet NSTextField*			networkSiteProxyPortTextField;
@property (weak) IBOutlet NSTextField*			networkSiteProxyErrorTextField;
@property (weak) IBOutlet NSButton*				networkSiteProxyScriptEditBtn;
//
@property (weak) IBOutlet NSButton*				developerTransformErrorHtmlCB;
@property (weak) IBOutlet NSButton*				developerCheckerboardPatternCB;
//
@property (weak) IBOutlet NSButton*				developerDebugFlagCB;
@property (weak) IBOutlet NSButton*				developerDebugVerboseCB;
@property (weak) IBOutlet NSButton*				developerDebugHttpClientCB;
@property (weak) IBOutlet NSButton*				developerDebugHttpServerCB;
@property (weak) IBOutlet NSButton*				developerDebugNetworkConfigCB;
@property (weak) IBOutlet NSButton*				developerDebugFiltersCB;
@property (weak) IBOutlet NSButton*				developerDumpHttpClientCacheBtn;
@property (weak) IBOutlet NSTextField*			developerDumpHttpCacheMessageTextView;
@property (weak) IBOutlet NSBox*				developerInternalBox;

@property IBOutlet NSTextView*					developerUpdatesBetaTextView;
@property (weak) IBOutlet NSButton*				developerUpdatesBetaCB;
@property (weak) IBOutlet NSTextField*			developerUpdatesLastDateTextField;



- (void)startFileSystemEventListener;
- (void)stopFileSystemEventListener;

- (void)updateSelectedTabView;

- (void)updateTabViewSetup;
- (void)updateAutoUpdateViews;

- (void)updateTabViewFilters;
- (void)releaseFilterTabController;
- (void)checkReloadSubscriptionFilterData;

- (void)updateTabViewHistory;

- (void)updateTabViewSuspects;

- (void)updateTabViewNetwork;
- (void)updateTabViewDeveloper;

- (void)updateNetworkSettingsIfWanted;
- (void)updateNetworkSettingsWithFeedbackIfUnchanged:(BOOL)feedbackIfUnchanged;

- (void)workspaceSessionDidBecomeActiveNotification:(NSNotification*)notification;
- (void)workspaceSessionDidResignActiveNotification:(NSNotification*)notification;
- (void)checkEditFilterNotification:(NSNotification*)notification;

- (void)setCheckbox:(NSButton*)button withXPath:(NSString*)xpath withEnabled:(BOOL)enabled;
@end

@implementation GlimmerBlockerPrefsPane

static NSString *globalCopyrightHtml =
@"<style type='text/css'>p { margin: 0px; text-align: left; font: 9.0px 'Lucida Grande'; }</style>"
@"<p>Copyright 2008-2015 Peter Speck. GlimmerBlocker is licensed under <a href='http://www.gnu.org/licenses/gpl.html'>GPLv3</a> and is free software.</p>"
@"<p>&nbsp;</p>"
@"<p>Thanks to <a href='mailto:elpincho@telefonica.net'>Pedro</a> for permission to use the Tangerine icon.</p>"
@"<p>The <a href='http://www.mozilla.org/rhino/'>Rhino</a> Javascript runtime is developed by mozilla.org, <a href='http://www.dnsjava.org/'>dnsjava</a> by Brian Wellington, <a href='http://www.oac.uci.edu/indiv/ehood/man2html.html'>man2html</a> by Earl Hood.</p>"
;

static NSString *betaVersionsDescriptionHtml =
@"<style type='text/css'>p { margin: 0px; text-align: left; font: 9.0px 'Lucida Grande'; }</style>"
@"<p>Help test new features."
@"<br>"
@" Send bug reports to <a href='mailto:feedback@glimmerblocker.org'>feedback@glimmerblocker.org</a>"
@"<br>"
@" Beta-releases might have bugs!"
@"</p>"
;


-(instancetype)initWithBundle:(NSBundle*)bundle
{
	DebugNSLog(@"NSAppKitVersionNumber = %f", NSAppKitVersionNumber);
	if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_5) {
		NSLog(@"GlimmerBlocker refusing to run under Mac OS X 10.5 and earlier.");
		// todo: show alert?
		return NULL;
	}
	[AutoZap initGlobals];
	if (!(self = [super initWithBundle:bundle]))
		return NULL;
	if (getenv("NSZombieEnabled"))
		NSLog(@"NSZombieEnabled enabled!");
	if (getenv("NSAutoreleaseFreedObjectCheckEnabled"))
		NSLog(@"NSAutoreleaseFreedObjectCheckEnabled enabled!");
	srandom((int)time(NULL));
	NSExceptionHandler* exHandler = [NSExceptionHandler defaultExceptionHandler];
	[exHandler setExceptionHandlingMask:NSLogAndHandleEveryExceptionMask];
	[exHandler setDelegate:self];
	[NSDateFormatter setDefaultFormatterBehavior:NSDateFormatterBehavior10_4];
	_glimmerBlockerVersionText = [bundle infoDictionary][@"CFBundleVersion"];
	return self;
}

- (void)dealloc
{
	[self stopFileSystemEventListener];
	[_prefs saveIfDirty:self];
	[_history notifyBecameHidden];
	[_suspects notifyBecameHidden];
}

- (BOOL)checkInstallation
{
	if (_installationHelper)
		return NO;
	_installationHelper = [[InstallationHelper alloc] initWithCopyrightMessage:globalCopyrightHtml
															withPreferencePane:self];
	if (![_installationHelper validate:_glimmerBlockerVersionText]) {
		[self stopFileSystemEventListener];
		return NO;
	}
	_installationHelper = NULL;
	return YES;
}

- (void)assignMainView
{
	NSAssert(_mainPrefsPaneView, @"Missing _mainPrefsPaneView");
	if (![self checkInstallation])
		return;
	[self setMainView:_mainPrefsPaneView];
}

- (void)setTextView:(NSTextView*)view html:(NSString*)html
{
	NSAttributedString *as = [[NSAttributedString alloc] initWithHTML:[html dataUsingEncoding:NSUTF8StringEncoding] documentAttributes:NULL];
	[[view textStorage] setAttributedString:as];
}

- (void)mainViewDidLoad
{
	[self setTextView:_setupCopyrightTextView html:globalCopyrightHtml];
	[self setTextView:_developerUpdatesBetaTextView html:betaVersionsDescriptionHtml];
	if (_installationHelper)
		return;
	NSAssert(_mainPrefsPaneView, @"_mainPrefsPaneView");
	//
	DebugNSLog(@"mainViewDidLoad, has-window = %d", !![_mainPrefsPaneView window]);
	_alertHelper = [[AlertHelper alloc] initWindowView:_mainPrefsPaneView withMainBundle:[self bundle]];
	[ProxyScriptEditor initProxyScriptEditor];
	_prefs = [Prefs prefsWithAlertHelper:_alertHelper
				  withSFAuthorizationView:_authView];
	_comm = [[ServerCommunication alloc] initWithPrefs:_prefs withMainBundle:[self bundle]];
	_filterTabController = [[FilterTabController alloc] initWithServerCommunication:_comm
																  withContainerView:_filtersContainerView
																		withTabView:_tabView
															 withFiltersTabViewItem:_filtersTabViewItem
																		 withBundle:[self bundle]];
	_history = [[HistoryController alloc] initWithFilterTabController:_filterTabController
													withContainerView:_historyContainerView
														   withBundle:[self bundle]];
	_suspects = [[SuspectsController alloc] initWithFilterTabController:_filterTabController
													  withContainerView:_suspectsContainerView
															 withBundle:[self bundle]];
	// minimize window height to ensure users can click the SFAuthorizationView button on 800*600 monitors.
	CGFloat shrink = 675 - [[NSScreen mainScreen] frame].size.height;
	if (shrink > 0) {
		NSSize sz = [_mainPrefsPaneView frame].size;
		sz.height -= shrink;
		[_mainPrefsPaneView setFrameSize:sz];
	}
	[_tabView setDelegate:self];
	NSAssert(_setupEnableGlimmerBlockerCB, @"t1EnableCB");
	[_setupCopyrightTextView setEditable:NO]; // easier to have YES in IB, as it won't otherwise allow editing in IB.
	[_developerUpdatesBetaTextView setEditable:NO]; // easier to have YES in IB, as it won't otherwise allow editing in IB.
	[_developerDumpHttpCacheMessageTextView setHidden:YES];
	//
	NSString* s = [_prefs stringForXPath:@"iu/@selected-tab"];
	if (s) {
		DebugNSLog(@"previous selected tab: %@", s);
		NSInteger idx = [_tabView indexOfTabViewItemWithIdentifier:s];
		if (idx == NSNotFound)
			s = NULL;
		else
			[_tabView selectTabViewItemAtIndex:idx];
	}
	if (!s)
		[_tabView selectFirstTabViewItem:self];
	//
	NSAssert(_authView, @"_authView");
	[_authView setString:glimmerAuthString];
	[_authView setDelegate:self];
	[_authView updateStatus:self]; // force immediate update
	[_authView setAutoupdate:YES];
//#ifndef DEBUG
//	[_developerInternalBox setHidden:YES];
//#endif
	//DebugNSLog(@"auth state: %d", [_authView authorizationState]);
	NSNotificationCenter* nc = [[NSWorkspace sharedWorkspace] notificationCenter];
	[nc addObserver:self
		   selector:@selector(workspaceSessionDidBecomeActiveNotification:)
			   name:NSWorkspaceSessionDidBecomeActiveNotification
			 object:nil];
	[nc addObserver:self
		   selector:@selector(workspaceSessionDidResignActiveNotification:)
			   name:NSWorkspaceSessionDidResignActiveNotification
			 object:nil];
	if ([_prefs zapRulePreviewElement] && [_prefs hasGlimmerAuth])
		[_comm saveDirtyPrefsAndNotify];
}

#pragma mark - delegate methods

- (void)willSelect
{
	if (_installationHelper)
		return;
	DebugNSLog(@"Our prefs panel is about to be selected.");
}

- (void)didSelect
{
	if (![self checkInstallation])
		return;
	DebugNSLog(@"Our prefs panel is selected.");
	DebugNSLog(@"Our window: %@", [_tabView window]);
	[_comm performPostLoadPreferencePanelCheck];
	if ([_prefs boolForXPath:@"proxy-server/@enabled"] && ![_comm isServerRunning]) {
		NSLog(@"GlimmerBlockerProxy not running. Starts it.");
		[_comm startServer];
		for (int i = 0; i < 10; i++) {
			@try {
				NSError* __autoreleasing error = NULL;
				NSString* s = [_comm curl:@"/test-running" withErrorRef:&error];
				if (!error && s) {
					NSLog(@"Server now running [%d]: %@", i, s);
					break;
				}
				NSLog(@"GB server not started yet [%@]: %@", error, s);
			}
			@catch (NSException* ex) {
				NSLog(@"GB server not started yet [%d]: %@", i, ex);
			}
			sleep(1); // ugly, but shouldn't take (too) long.
		}
	}
	[_filterTabController postInit];
	if (!_hasAddedWindowNotifications) {
		_hasAddedWindowNotifications = YES;
		DebugNSLog(@"adds WindowNotifications: %@", NSApp);
		NSNotificationCenter* nc = [[NSWorkspace sharedWorkspace] notificationCenter];
		[nc addObserver:self
			   selector:@selector(checkEditFilterNotification:)
				   name:NSWindowDidBecomeKeyNotification
				 object:[_tabView window]];
		[nc addObserver:self
			   selector:@selector(checkEditFilterNotification:)
				   name:NSApplicationDidUnhideNotification
				 object:NSApp];
		[nc addObserver:self
			   selector:@selector(checkEditFilterNotification:)
				   name:NSApplicationDidBecomeActiveNotification
				 object:NSApp];
	}
	[self checkEditFilterNotification:NULL];
	[self updateSelectedTabView];
	[self startFileSystemEventListener];
	[self checkReloadSubscriptionFilterData];
}

- (void)willUnselect
{
	if (_installationHelper)
		return;
	DebugNSLog(@"Our prefs panel is about to be un-selected.");
	DebugNSLog(@"Our window: %@", [_tabView window]);
}

- (void)didUnselect
{
	if (_installationHelper)
		return;
	DebugNSLog(@"Our prefs panel is now un-selected.");
	DebugNSLog(@"Our window: %@", [_tabView window]);
	[self stopFileSystemEventListener];
}

+ (void)openWebsiteInBrowser
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://glimmerblocker.org"]];
}

// -------------------------------------------------------------------------------
- (void)workspaceSessionDidBecomeActiveNotification:(NSNotification*)notification
{
	DebugNSLog(@"GB.workspaceSessionDidBecomeActiveNotification");
	if (![self checkInstallation])
		return;
	[_prefs forcedReload];
	[self checkEditFilterNotification:notification];
	[self updateSelectedTabView];
	if ([self isSelected])
		[self startFileSystemEventListener];
}

- (void)workspaceSessionDidResignActiveNotification:(NSNotification*)notification
{
	DebugNSLog(@"GB.workspaceSessionDidResignActiveNotification");
	[self stopFileSystemEventListener];
	[self releaseFilterTabController];
	[_history notifyBecameHidden];
	[_suspects notifyBecameHidden];
	[_prefs saveIfDirty:self];
}

static void fileSystemEventCallback(ConstFSEventStreamRef streamRef, void *clientCallBackInfo, size_t numEvents,
									void *eventPaths, const FSEventStreamEventFlags eventFlags[], const FSEventStreamEventId eventIds[])
{
#ifdef DEBUG_FSEVENTS
	NSLog(@"Got %d filesystem changes:  0x%lx\n", (int)numEvents, (long)clientCallBackInfo);
	char **paths = eventPaths;
	for (size_t i=0; i<numEvents; i++)
		NSLog(@"%ld = %s\n", eventFlags[i], paths[i]);
#endif
	GlimmerBlockerPrefsPane* pp = (__bridge GlimmerBlockerPrefsPane*)clientCallBackInfo;
	[pp checkReloadSubscriptionFilterData];
}

- (void)stopFileSystemEventListener
{
	if (!_fsEventStreamRef)
		return;
#ifdef DEBUG_FSEVENTS
	DebugNSLog(@"Disposes FSEvent listener");
#endif
	FSEventStreamStop(_fsEventStreamRef);
	FSEventStreamInvalidate(_fsEventStreamRef);
	FSEventStreamRelease(_fsEventStreamRef);
	_fsEventStreamRef = NULL;
}

- (void)startFileSystemEventListener
{
	if (_fsEventStreamRef)
		return;
#ifdef DEBUG_FSEVENTS
	DebugNSLog(@"Creates FSEvent listener: 0x%lx", self);
#endif
	CFStringRef mypath = CFSTR("/Library/GlimmerBlocker/Filter subscriptions");
	CFArrayRef pathsToWatch = CFArrayCreate(NULL, (const void **)&mypath, 1, NULL);
	FSEventStreamContext context = {0, (__bridge void *)(self), NULL, NULL, NULL};
	CFAbsoluteTime latency = 0.5;
	_fsEventStreamRef = FSEventStreamCreate(NULL, fileSystemEventCallback, &context, pathsToWatch, kFSEventStreamEventIdSinceNow,
											latency, kFSEventStreamCreateFlagNone);
	CFRelease(pathsToWatch);
	if (!_fsEventStreamRef) {
		NSLog(@"Failed creating FSEventStream");
		return;
	}
	CFRunLoopRef mainLoop = [[NSRunLoop currentRunLoop] getCFRunLoop];
	FSEventStreamScheduleWithRunLoop(_fsEventStreamRef, mainLoop, kCFRunLoopDefaultMode);
	FSEventStreamStart(_fsEventStreamRef);
}

// -------------------------------------------------------------------------------
- (void)checkEditFilterNotification:(NSNotification*)notification
{
	DebugNSLog(@"checkEditFilterNotification: %@", [notification name]);
	NSString* path = [NSString stringWithFormat:@"/tmp/gb-edit-filter-%d", getuid()];
	NSFileManager* fm = [NSFileManager defaultManager];
	if (![fm isReadableFileAtPath:path])
		return;
	NSData* data = [NSData dataWithContentsOfURL:[NSURL fileURLWithPath:path]];
	if (!data)
		return;
	NSInteger filterId = [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] intValue];
	DebugNSLog(@"filter-id from gb-edit-filter: %ld", (long)filterId);
	if ([[_tabView window] attachedSheet]) {
		DebugNSLog(@"Can't show filter: has sheet open.");
		return;
	}
	[_tabView selectTabViewItem:_filtersTabViewItem];
	[self updateSelectedTabView];
	DebugNSLog(@" updates.");
	Filter* filter = [_filterTabController selectFilterById:filterId];
	if (!filter)
		DebugNSLog(@"Unknown filter.");
	[fm removeItemAtPath:path error:NULL];
	[filter startEditingSubscription:YES];
}

// -------------------------------------------------------------------------------
- (void)updateSelectedTabView
{
	NSString *fane = [[_tabView selectedTabViewItem] identifier];
	if (![fane isEqual:@"suspects"])
		[_suspects notifyBecameHidden];
	if (![fane isEqual:@"history"])
		[_history notifyBecameHidden];
	//
	if ([fane isEqual:@"setup"])
		[self updateTabViewSetup];
	else if ([fane isEqual:@"filters"])
		[self updateTabViewFilters];
	else if ([fane isEqual:@"history"])
		[self updateTabViewHistory];
	else if ([fane isEqual:@"suspects"])
		[self updateTabViewSuspects];
	else if ([fane isEqual:@"network"])
		[self updateTabViewNetwork];
	else if ([fane isEqual:@"developer"])
		[self updateTabViewDeveloper];
	else
		NSLog(@"unknown tabViewItem identifier = '%@'", fane);
}

- (void)notifyUpdatedAuthorizationState
{
	[_prefs notifyUpdatedAuthorizationState]; // must be first as it updates cached state.
	[self updateSelectedTabView];
	[_filterTabController notifyUpdatedAuthorizationState];
	[_history notifyUpdatedAuthorizationState];
	[_suspects notifyUpdatedAuthorizationState];
}

#pragma mark ------------------- settings pane

- (void)updateTabViewSetup
{
	BOOL hasAuth = [_prefs hasGlimmerAuth];
	[_setupEnableGlimmerBlockerCB setState:[_prefs boolForXPath:@"proxy-server/@enabled"]];
	[_setupEnableGlimmerBlockerCB setEnabled:hasAuth];
	[self updateAutoUpdateViews];
}

- (IBAction)setupEnableGlimmerBlockerAction:(id)sender
{
	BOOL enable = [_setupEnableGlimmerBlockerCB state];
	if (enable && ![_prefs save]) {
		NSLog(@"Can't save prefs, thus can't start proxy");
		[_setupEnableGlimmerBlockerCB setState:NO];
		return;
	}
	[_prefs setBool:enable forXPath:@"proxy-server/@enabled"];
	[_prefs markAsDirty];
	if (enable)
		[_comm startServer];
	else
		[_comm stopServer];
	[self updateNetworkSettingsIfWanted];
}

- (IBAction)setupConfirmNetworkSettingsAction:(id)sender
{
	BOOL allow = [sender state];
	[_prefs setBool:allow forXPath:@"network-settings/@confirm"];
	[_prefs markAsDirty];
}

- (IBAction)setupSyncNetworkSettingsNowAction:(id)sender
{
	[self updateNetworkSettingsWithFeedbackIfUnchanged:YES];
}

//=================================================================================
- (IBAction)setupUpdatesAutoAction:(id)sender
{
	[_prefs setBool:[sender state] forXPath:@"updater/@check-in-background"];
	[_prefs markAsDirty];
	[_comm updateUpdaterLaunchd];
}

- (void)postUpdateCheck
{
	[self performSelector:@selector(updateAutoUpdateViews) withObject:NULL afterDelay:5.0];
	[self performSelector:@selector(updateAutoUpdateViews) withObject:NULL afterDelay:15.0];
	[self performSelector:@selector(updateAutoUpdateViews) withObject:NULL afterDelay:25.0];
	[self performSelector:@selector(updateAutoUpdateViews) withObject:NULL afterDelay:60.0];
}

- (IBAction)setupUpdatesManualAction:(id)sender
{
	[_comm launchUpdaterApp];
	// keep the button visually pressed until the helper had a chance to launch,
	// so it doesn't seem like nothing happends.
	sleep(2);
	[self postUpdateCheck];
}

- (IBAction)setupUpdatesBetaAction:(id)sender
{
	[_prefs setBool:[sender state] forXPath:@"updater/@use-beta-versions"];
	[_prefs markAsDirty];
	[_comm updateUpdaterLaunchd];
}

- (void)updateAutoUpdateViews
{
	//DebugNSLog(@"updateAutoUpdateViews...");
	BOOL hasAuth = [_prefs hasGlimmerAuth];
	[self setCheckbox:_setupUpdatesAutoCB withXPath:@"updater/@check-in-background" withEnabled:hasAuth];
	[self setCheckbox:_developerUpdatesBetaCB withXPath:@"updater/@use-beta-versions" withEnabled:hasAuth];
	[_setupUpdatesNowBtn setEnabled:hasAuth];
	[_setupUpdatesNowBtn2 setEnabled:hasAuth];
	//
	NSFileManager* fm = [NSFileManager defaultManager];
	NSDictionary* dict = [fm attributesOfItemAtPath:LAST_UPDATE_CHECK_MARKER_PATH error:NULL];
	NSDate* lastCheckDate = dict[NSFileModificationDate];
	NSString* s;
	if (lastCheckDate) {
		NSDateFormatter* formatter = [[NSDateFormatter alloc] init];
		[formatter setDateStyle:NSDateFormatterShortStyle];
		[formatter setTimeStyle:NSDateFormatterShortStyle];
		s = [NSString stringWithFormat:@"Last check: %@", [formatter stringFromDate:lastCheckDate]];
	} else {
		s = @"Last check: none";
		[_developerUpdatesLastDateTextField setStringValue:s];
	}
	NSString* v = [NSString stringWithFormat:@"Installed version: %@", _glimmerBlockerVersionText];
	s = [NSString stringWithFormat:@"%@\n%@", v, s];
	[_setupUpdatesLastDateTextField setStringValue:s];
	[_developerUpdatesLastDateTextField setStringValue:s];
}

#pragma mark ------------------- filters

- (void)updateTabViewFilters
{
	BOOL running = [_comm isServerRunning];
	[_filtersMissingServerLabel setHidden:running];
	[_filtersContainerView setHidden:!running];
	[_filterTabController postInit];
	if (![_mainPrefsPaneView window]) // not fully initialized yet - can't display alerts.
		return;
	if (!running)
		return;
	[_filterTabController updateTabViewWithForceUpdate:NO];
}

- (void)releaseFilterTabController
{
	_filterTabController = NULL;
}

- (void)checkReloadSubscriptionFilterData
{
	if (_installationHelper)
		return;
	if (![_comm isServerRunning])
		return; // can't reload filters when the server isn't running.
	[_filterTabController postInit];
	for (Filter* filter in [_filterTabController filterArray])
		[filter checkReloadSubscriptionFilterData];
}

#pragma mark ------------------- history

- (void)updateTabViewHistory
{
	DebugNSLog(@"updateTabViewHistory");
	BOOL running = [_comm isServerRunning];
	[_historyMissingServer setHidden:running];
	[_historyContainerView setHidden:!running];
	if (![_mainPrefsPaneView window]) // not fully initialized yet - can't display alerts.
		return;
	if (!running)
		return;
	[_history notifyBecameVisible];
}

#pragma mark ------------------- suspects

- (void)updateTabViewSuspects
{
	DebugNSLog(@"updateTabViewSuspects");
	BOOL running = [_comm isServerRunning];
	[_suspectsMissingServer setHidden:running];
	[_suspectsContainerView setHidden:!running];
	if (![_mainPrefsPaneView window]) // not fully initialized yet - can't display alerts.
		return;
	if (!running)
		return;
	[_suspects notifyBecameVisible];
}

#pragma mark ------------------- network

- (void)updateTabViewNetwork
{
	[self updateAutoUpdateViews];
	BOOL hasAuth = [_prefs hasGlimmerAuth];
	[_networkPortNumberTextField setStringValue:[NSString stringWithFormat:@"%d", [_comm getPortNumber]]];
	[_networkPortNumberTextField setEnabled:hasAuth];
	[_networkAllowOtherComputersCB setState:[_prefs boolForXPath:@"proxy-server/@listen-wildcard"] ? 1 : 0];
	[_networkAllowOtherComputersCB setEnabled:hasAuth];
	[_networkConfirmNetworkSettingsCB setState:[_prefs boolForXPath:@"network-settings/@confirm"]];
	[_networkConfirmNetworkSettingsCB setEnabled:hasAuth];
	[_networkSyncNetworkSettingsNowBtn setEnabled:hasAuth];
	//
	NSString* host = [_prefs stringForXPath:@"network-settings/@site-proxy-host"];
	[_networkSiteProxyHostTextField setStringValue:(host ? host : @"")];
	NSString* port = [_prefs stringForXPath:@"network-settings/@site-proxy-port"];
	[_networkSiteProxyPortTextField setStringValue:(port ? port : @"")];
	ProxyType type = (ProxyType)[proxyTypeEnum indexFromXPath:@"network-settings/@site-proxy" withPrefs:_prefs];
	[_networkSiteProxyMatrix selectCellAtRow:type column:0];
	[_networkSiteProxyMatrix setEnabled:hasAuth];
	//DebugNSLog(@"proxy: %d   %@:%@", [_prefs boolForXPath:@"network-settings/@site-proxy"], host, port);
	//
	[self networkSiteProxyAction:self];
}

- (IBAction)networkPortNumberAction:(id)sender
{
	NSString *s = [_networkPortNumberTextField stringValue];
	DebugNSLog(@"networkPortNumberAction: %@", s);
	NSUInteger i;
	int port = 0;
	for (i = 0; i < [s length]; i++) {
		unichar ch = [s characterAtIndex:i];
		if (ch < '0' || ch > '9') {
			NSLog(@"Invalid port number: %@", s); // todo: error alert
			return;
		}
		port = port * 10 + ch - '0';
	}
	int minPort = 1024;
	int maxPort = 65535;
	if (port < minPort || port > maxPort) {
		NSString* title = [NSString stringWithFormat:@"Invalid port number: %d", port];
		NSString* help = [NSString stringWithFormat:@"Must be between %d and %d", minPort, maxPort];
		[_alertHelper prepareCriticalAlertWithTitle:title
								withInformativeText:help];
		[_alertHelper showPreparedAlert];
		[_networkPortNumberTextField setStringValue:[NSString stringWithFormat:@"%d", [_comm getPortNumber]]];
		return;
	}
	int oldPort = [_comm getPortNumber];
	if (oldPort == port)
		return;
	DebugNSLog(@"Port number: %d -> %d", oldPort, port);
	[_prefs setInt:port forXPath:@"proxy-server/@listen-port"];
	[_prefs markAsDirty];
	if (![_comm isServerRunning])
		return;
	[_comm saveDirtyPrefsAndNotify];
	[self updateNetworkSettingsIfWanted];
}

- (IBAction)networkAllowOtherComputersAction:(id)sender
{
	BOOL allow = [sender state];
	[_prefs setBool:allow forXPath:@"proxy-server/@listen-wildcard"];
	[_prefs markAsDirty];
	[_comm saveDirtyPrefsAndNotify];
}

- (void)notifyProxyScriptEditorCommitted:(ProxyScriptEditor*)editor
{
	[self networkSiteProxyAction:self];
}

- (IBAction)networkSiteProxyScriptEditAction:(id)sender
{
	ProxyScriptEditor *editor = [[ProxyScriptEditor alloc] initWithServerCommunication:_comm
																			withBundle:[self bundle]
																		  withDelegate:self];
	[AutoZap add:editor];
}

- (BOOL)validateStaticProxyHost:(NSString*)host withPort:(NSString*)port
{
	NSString* msg = NULL;
	if ([_mainPrefsPaneView window] && [_comm isServerRunning]) {
		// can only validate when _comm is fully initialized
		NSString* pathArgs = [NSString stringWithFormat:@"/site-proxy/validate?host=%@&port=%@",
							  [GBDebug percentEscapeUrlParameterValue:host],
							  [GBDebug percentEscapeUrlParameterValue:port]];
		XmlDoc* doc = [_comm curlToXml:pathArgs withActionDescription:NULL];
		if (doc)
			msg = [doc stringForXPath:@"@proxy-error"];
		else
			msg = @"Problem validating proxy hostname.";
	}
	[_networkSiteProxyErrorTextField setStringValue:(msg ? msg : @"")];
	return ![msg length];
}

- (BOOL)validateProxyJavascript
{
	NSString* msg = NULL;
	if ([_mainPrefsPaneView window] && [_comm isServerRunning]) {
		// can only validate when _comm is fully initialized
		NSString* js = [_prefs stringForXPath:@"network-settings/proxy-script"];
		NSString* pathArgs = [NSString stringWithFormat:@"/site-proxy/validate?javascript=%@",
							  [GBDebug percentEscapeUrlParameterValue:js]];
		XmlDoc* doc = [_comm curlToXml:pathArgs withActionDescription:NULL];
		if (doc)
			msg = [doc stringForXPath:@"@proxy-error"];
		else
			msg = @"Problem validating proxy javascript.";
	}
	[_networkSiteProxyErrorTextField setStringValue:(msg ? msg : @"")];
	return ![msg length];
}

- (IBAction)networkSiteProxyAction:(id)sender;
{
	ProxyType oldType = (ProxyType)[proxyTypeEnum indexFromXPath:@"network-settings/@site-proxy" withPrefs:_prefs];
	NSString* oldHost = [_prefs stringForXPath:@"network-settings/@site-proxy-host"];
	NSString* oldPort = [_prefs stringForXPath:@"network-settings/@site-proxy-port"];
	NSString* host = [[_networkSiteProxyHostTextField stringValue] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
	NSString* port = [[_networkSiteProxyPortTextField stringValue] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
	BOOL hasAuth = [_prefs hasGlimmerAuth];
	ProxyType type = (ProxyType)[_networkSiteProxyMatrix selectedRow];
	//DebugNSLog(@"networkSiteProxyAction: type = %d,  static = %@:%@", type, host, port);
	[_networkSiteProxyScriptEditBtn setEnabled:(type == JavascriptProxy && hasAuth)];
	NSColor* c = (type == StaticProxy) ? [NSColor blackColor] : [NSColor grayColor];
	[_networkSiteProxyHostLabel setTextColor:c];
	[_networkSiteProxyPortLabel setTextColor:c];
	[_networkSiteProxyHostTextField setEnabled:(type == StaticProxy && hasAuth)];
	[_networkSiteProxyPortTextField setEnabled:(type == StaticProxy && hasAuth)];
	if (type == StaticProxy && sender == _networkSiteProxyMatrix) {
		[[_mainPrefsPaneView window] performSelectorOnMainThread:@selector(makeFirstResponder:)
													 withObject:_networkSiteProxyHostTextField
												  waitUntilDone:NO];
	}
	if (type == StaticProxy && ![self validateStaticProxyHost:host withPort:port])
		return;
	if (type == JavascriptProxy && ![self validateProxyJavascript])
		return;
	[_networkSiteProxyErrorTextField setStringValue:@""];
	BOOL changed = (type != oldType);
	if (type == StaticProxy && (![host isEqual:oldHost] || ![port isEqual:oldPort])) {
		[_prefs setString:host forXPath:@"network-settings/@site-proxy-host"];
		[_prefs setString:port forXPath:@"network-settings/@site-proxy-port"];
		changed = true;
		DebugNSLog(@"Updated static site-wide proxy:  %@:%@  ->  %@:%@", oldHost, oldPort, host, port);
	}
	if (!changed)
		return;
	[_prefs setString:[proxyTypeEnum nameFromIndex:type] forXPath:@"network-settings/@site-proxy"];
	[_prefs markAsDirty];
	[_comm saveDirtyPrefsAndNotify];
}


#pragma mark ------------------- developer

- (void)updateTabViewDeveloper
{
	[self updateAutoUpdateViews];
	BOOL hasAuth = [_prefs hasGlimmerAuth];
	[_developerDumpHttpClientCacheBtn setEnabled:hasAuth];
	[self setCheckbox:_developerDebugFlagCB withXPath:@"debug/@enabled" withEnabled:hasAuth];
	BOOL enabled = hasAuth && [_prefs boolForXPath:@"debug/@enabled"];
	[self setCheckbox:_developerDebugVerboseCB withXPath:@"debug/@verbose" withEnabled:enabled];
	[self setCheckbox:_developerDebugHttpClientCB withXPath:@"debug/@http-client" withEnabled:enabled];
	[self setCheckbox:_developerDebugHttpServerCB withXPath:@"debug/@http-server" withEnabled:enabled];
	[self setCheckbox:_developerDebugNetworkConfigCB withXPath:@"debug/@network-config" withEnabled:enabled];
	[self setCheckbox:_developerDebugFiltersCB withXPath:@"debug/@filters" withEnabled:enabled];
	//
	[self setCheckbox:_developerTransformErrorHtmlCB withXPath:@"debug/@transform-html-box" withEnabled:hasAuth];
	[self setCheckbox:_developerCheckerboardPatternCB withXPath:@"debug/@checkerboard-pattern" withEnabled:hasAuth];
}

- (IBAction)developerDebugFlagsAction:(id)sender
{
	[_prefs setBool:[_developerDebugFlagCB state] forXPath:@"debug/@enabled"];
	[_prefs setBool:[_developerDebugVerboseCB state] forXPath:@"debug/@verbose"];
	[_prefs setBool:[_developerDebugHttpClientCB state] forXPath:@"debug/@http-client"];
	[_prefs setBool:[_developerDebugHttpServerCB state] forXPath:@"debug/@http-server"];
	[_prefs setBool:[_developerDebugNetworkConfigCB state] forXPath:@"debug/@network-config"];
	[_prefs setBool:[_developerDebugFiltersCB state] forXPath:@"debug/@filters"];
	[_prefs setBool:[_developerTransformErrorHtmlCB state] forXPath:@"debug/@transform-html-box"];
	[_prefs setBool:[_developerCheckerboardPatternCB state] forXPath:@"debug/@checkerboard-pattern"];
	[self updateTabViewDeveloper];
	[_prefs markAsDirty];
	[_comm saveDirtyPrefsAndNotify];
}

- (IBAction)developerShowLogAction:(id)sender
{
	[_comm openServerLogInConsole];
}

- (IBAction)developerDumpHttpClientCacheAction:(id)sender
{
	[_developerDumpHttpCacheMessageTextView setHidden:NO];
	[_comm curl:@"!/http-cache/dump"];
}

#pragma mark ------------------- utils

- (IBAction)debugInternalSystemLog:(id)sender
{
	[GBDebug executeCommandlineTool:@"open" withArgs:@[@"-a", @"Console", @"/var/log/system.log"]];
}

- (void)makeUrlTest:(NSString*)url
{
	NSLog(@"Makes NSURL request to %@", url);
	@try {
		NSMutableURLRequest* req = [[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:url]];
		[req setCachePolicy:NSURLRequestReloadIgnoringLocalCacheData];
		[req setTimeoutInterval:5.0];
		NSError *__autoreleasing error = NULL;
		NSURLResponse *response = NULL;
		NSData *data = [NSURLConnection sendSynchronousRequest:req returningResponse:&response error:&error];
		NSLog(@"data: %ld bytes: %@", [data length], data);
		NSHTTPURLResponse *hr = (NSHTTPURLResponse*)response;
		if (hr)
			NSLog(@"response: %ld = %@", (long)[hr statusCode], [hr allHeaderFields]);
		if (error)
			NSLog(@"error: %@", error);
	}
	@catch (NSException* ex) {
		NSLog(@"NSURLConnection failed with exception: %@", ex);
		[GBDebug debugDumpExceptionStackframe:ex];
	}
}

- (IBAction)debugInternalRunTcpTests:(id)sender
{
	NSLog(@"------------------------------- START OF DEBUG");
	NSString *url = [NSString stringWithFormat:@"http://127.0.0.1:%d", [_comm getPortNumber]];
	NSArray *cArgs = @[@"--silent", @"--dump-header", @"-", @"--connect-timeout", @"10", @"--max-time", @"10", url];
	[GBDebug executeCommandlineTool:@"curl"
						   withArgs:cArgs];
	[GBDebug executeCommandlineTool:@"ps"
						   withArgs:@[@"wwwU", @"_glimmerblocker"]];
	[GBDebug executeCommandlineTool:@"ls"
						   withArgs:@[@"-l", @"/Library/GlimmerBlocker"]];
	[GBDebug executeCommandlineTool:@"ls"
						   withArgs:@[@"-l", @"/Library/PreferencePanes/GlimmerBlocker.prefPane/Contents"]];
	[GBDebug executeCommandlineTool:@"du"
						   withArgs:@[@"-hd1", @"/Library/PreferencePanes/GlimmerBlocker.prefPane/Contents"]];
	[GBDebug executeCommandlineTool:@"id"
						   withArgs:@[@"_glimmerblocker"]];
	[GBDebug executeCommandlineTool:@"dscl"
						   withArgs:@[@".", @"-read", @"/Users/_glimmerblocker"]];
	[GBDebug executeCommandlineTool:@"dscl"
						   withArgs:@[@".", @"-read", @"/Groups/_glimmerblocker"]];
	[GBDebug executeCommandlineTool:[GBDebug proxyBinaryPath]
						   withArgs:@[@"--main-class", @"org.glimmerblocker.proxy.TestJava"]];
	[self makeUrlTest:url];
	[self makeUrlTest:[url stringByAppendingString:@"/=gb=/-public-/hello"]];
	NSString *perl = @"$| = 1; foreach (1..5) { print \"stdio-$_\\n\"; Time::HiRes::sleep(0.1);; print STDERR \"stderr-$_\\n\"; Time::HiRes::sleep(0.1);}";
	[GBDebug executeCommandlineTool:@"perl"
						   withArgs:@[@"-MTime::HiRes", @"-e", perl]];
	NSLog(@"------------------------------- END OF DEBUG");
}

#pragma mark ------------------- utils

- (NSString*)controlAlternatingRowBackgroundColorsAsUrlParam
{
	NSArray* colors = [NSColor controlAlternatingRowBackgroundColors];
	NSMutableString* sb = [NSMutableString stringWithCapacity:40];
	NSColorSpace* sRGB = [NSColorSpace sRGBColorSpace];
	for (NSUInteger i = 0; i < [colors count]; i++) {
		if ([sb length])
			[sb appendString:@";"];
		else
			[sb appendString:@"arbc="];
		CGFloat r, g, b, a;
		NSColor* clr = [colors[i] colorUsingColorSpace:sRGB];
		// why can't I get the rgb when the color is converted to sRGB?
		// *** -getRed:green:blue:alpha: not defined for the
		//				NSColor NSCustomColorSpace sRGB IEC61966-2.1 colorspace 1 1 1 1;
		//				need to first convert colorspace.
		[clr getRed:&r green:&g blue:&b alpha:&a];
		[sb appendFormat:@"rgba(%f,%f,%f,%f)", r, g, b, a];
	}
	return sb;
}

#pragma mark ------------------- utils

- (void)updateNetworkSettingsIfWanted
{
	[self updateNetworkSettingsWithFeedbackIfUnchanged:NO];
}

- (void)updateNetworkSettingsWithFeedbackIfUnchanged:(BOOL)feedbackIfUnchanged
{
	BOOL confirm = [_prefs boolForXPath:@"network-settings/@confirm"];
	NetworkSettingsUpdaterUI* updater = [[NetworkSettingsUpdaterUI alloc]
										  initWithServerCommunication:_comm
										  withWantEnable:[_comm isServerRunning]];
	[updater updateSettingsWithConfirm:confirm withFeedbackIfUnchanged:feedbackIfUnchanged];
}

- (void)updateAuthRightsDefinition
{
	if (_hasUpdatedAuthRightsDefinition)
		return;
	SFAuthorization* auth = [_authView authorization];
	if (!auth) {
		DebugNSLog(@"updateAuthDefinition: Has no SFAuthorization");
		return;
	}
	AuthorizationRef authRef = [auth authorizationRef];
	if (!authRef) {
		DebugNSLog(@"updateAuthDefinition: [auth authorizationRef] returned null");
		return;
	}
	OSStatus err = AuthorizationRightGet(glimmerAuthString, NULL);
	if (err != errAuthorizationDenied) {
		_hasUpdatedAuthRightsDefinition = YES;
		//DebugNSLog(@"Has authright definition.");
		//err = AuthorizationRightRemove(authRef, glimmerAuthString);
		//DebugNSLog(@"AuthorizationRightRemove returned %d", err);
		return;
	}
	DebugNSLog(@"Missing authright definition.");
	CFURLRef cfBundleUrl = (__bridge CFURLRef)[NSURL fileURLWithPath:[[self bundle] bundlePath]];
	CFBundleRef cfBundle = CFBundleCreate(kCFAllocatorDefault, cfBundleUrl);
	NSDictionary* rightDefinition = @{@"class": @"user",
									 @"comment": @"Just a comment",
									 @"group": @"admin",
									 @"shared": @YES,
									 @"timeout": @(365 * 24 * 60 * 60)};
	err = AuthorizationRightSet(authRef,
								glimmerAuthString,
								(__bridge CFTypeRef)(rightDefinition),
								CFSTR("GlimmerBlocker authentication"),
								cfBundle,
								NULL); // localeTableName
	DebugNSLog(@"AuthorizationRightSet returned: %d", err);
	CFRelease(cfBundle);
	_hasUpdatedAuthRightsDefinition = !err;
}

- (void)setCheckbox:(NSButton*)button withXPath:(NSString*)xpath withEnabled:(BOOL)enabled
{
	BOOL state = [_prefs boolForXPath:xpath];
	[button setState:state ? NSOnState : NSOffState];
	[button setEnabled:enabled];
}

@end

@implementation GlimmerBlockerPrefsPane(NSTabViewDelegate)

- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
	if (_installationHelper)
		return;
	[self updateSelectedTabView];
	[_prefs setString:[tabViewItem identifier] forXPath:@"iu/@selected-tab"];
	if ([_prefs hasGlimmerAuth])
		[_prefs markAsDirty];
}

@end

@implementation GlimmerBlockerPrefsPane(NSExceptionHandlerDelegate)

- (BOOL)exceptionHandler:(NSExceptionHandler *)sender
   shouldHandleException:(NSException *)exception
					mask:(NSUInteger)aMask;
{
	// http://www.mactech.com/articles/mactech/Vol.19/19.12/CocoaDebug/
	NSLog(@"shouldHandleException '%@', reason: %@", [exception name], [exception reason]);
	[GBDebug debugDumpExceptionStackframe:exception];
	return YES;
}

- (BOOL)shouldDisplayException:(NSException *)exception;
{
	NSString* name = [exception name];
	NSString* reason = [exception reason];
	NSLog(@"SHOULD DISPLAY exception '%@', reason: %@", name, reason);
	return ![name isEqualToString:@"NSImageCacheException"]
	&& ![name isEqualToString:@"GIFReadingException"]
	&& ![name isEqualToString:@"NSRTFException"]
	&& !([name isEqualToString:@"NSInternalInconsistencyException"] && [reason hasPrefix:@"lockFocus"]);
}
@end


@implementation GlimmerBlockerPrefsPane(SFAuthorizationViewDelegate)

- (void)authorizationViewDidAuthorize:(SFAuthorizationView *)view
{
	DebugNSLog(@"GlimmerBlocker got authorizationViewDidAuthorize.");
	if (_installationHelper)
		return;
	[self updateAuthRightsDefinition];
	[self notifyUpdatedAuthorizationState];
	if (![_prefs hasGlimmerAuth]) {
		DebugNSLog(@"authorizationViewDidAuthorize: auth disappeared");
		return;
	}
}

- (void)authorizationViewDidDeauthorize:(SFAuthorizationView *)view
{
	DebugNSLog(@"GlimmerBlocker got authorizationViewDidDeauthorize.");
	if (_installationHelper)
		return;
	[self notifyUpdatedAuthorizationState];
}

@end
