/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import "NetworkSettingsUpdaterUI.h"
#import "GBDebug.h"
#import "ServerCommunication.h"
#import "AlertHelper.h"
#import "Prefs.h"

@interface NetworkSettingsUpdaterUI()

@property Prefs* prefs;
@property AlertHelper* alertHelper;

- (void)confirmUpdateAlertDidEndWithReturnCode:(NSInteger)returnCode;

@end

@implementation NetworkSettingsUpdaterUI

- (instancetype)initWithServerCommunication:(ServerCommunication*)comm
				   withWantEnable:(BOOL)wantEnable;
{
	if ((self = [super initWithProxyPort:[comm getPortNumber] withWantEnable:wantEnable])) {
		_prefs = [comm prefs];
		_alertHelper = [_prefs alertHelper];
	}
	return self;
}


- (SCPreferencesRef)createSCPreferencesRef:(BOOL)commit
{
	CFStringRef appName = CFSTR("GlimmerBlocker");
	if (commit)
		return SCPreferencesCreateWithAuthorization(NULL, appName, NULL, [_prefs authRef]);
	else
		return SCPreferencesCreate(NULL, appName, NULL);
}

- (void)updateSettingsWithConfirm:(BOOL)confirm  withFeedbackIfUnchanged:(BOOL)feedbackIfUnchanged
{
	if (![_prefs hasGlimmerAuth]) {
		NSLog(@"Can't change network settings without auth. Should never happend.");
		return;
	}
	BOOL ok = [self updateSettingsWithCommit:NO];
	if (ok) {
		DebugNSLog(@"Network settings don't need update.");
		if (feedbackIfUnchanged) {
			[_alertHelper prepareInformativeAlertWithTitle:@"No changes required to proxy settings"
									   withInformativeText:NULL];
			[_alertHelper showPreparedAlert];
		}
		return;
	}
	NSMutableString* sb = [NSMutableString stringWithCapacity:50];
	for (NSString* s in self.actionDescription) {
		if ([sb length])
			[sb appendString:@"\n"];
		[sb appendString:s];
		DebugNSLog(@"%@", s);
	}
	if (confirm) {
		[_alertHelper prepareWarningAlertWithTitle:@"Update proxy settings in Network?"
							   withInformativeText:sb];
		[_alertHelper setDelegate:self
					 withSelector:@selector(confirmUpdateAlertDidEndWithReturnCode:)];
		[_alertHelper addButtonWithTitle:@"Yes"];
		[_alertHelper addButtonWithTitle:@"No"];
		[_alertHelper showPreparedAlert];
		return;
	}
	ok = [self updateSettingsWithCommit:YES];
	if (!ok)
		return;
	ok = [self updateSettingsWithCommit:NO]; // check the settings was fully updated.
	if (!ok)
		NSLog(@"GlimmerBlocker failed updating network settings.");
}

- (void)confirmUpdateAlertDidEndWithReturnCode:(NSInteger)returnCode
{
	if (returnCode == NSAlertFirstButtonReturn)
		[self updateSettingsWithConfirm:NO withFeedbackIfUnchanged:NO];
}

@end
