/* Copyright (C) 2009-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import org.glimmerblocker.proxy.globals.log.DatePrefixedLogger;
import org.glimmerblocker.proxy.globals.log.Logger;
import org.glimmerblocker.proxy.globals.log.PrefixedLogger;
import org.jetbrains.annotations.Nullable;
import dk.vitality.lib.foundation.exception.ExceptionUtils;

/**
 * This server response with non-conforming responses to test how GB handles those.
 */
public class TestServer implements Logger
{
	private Logger logger;
	int nextConnectionNumber = 100;

	public static void main(String[] args)
	{
		new TestServer(new DatePrefixedLogger());
	}

	/**
	 * Can be called from GlimmerGlobals too.
	 */
	public TestServer(Logger parentLogger)
	{
		this.logger = new PrefixedLogger(parentLogger, "TestServer");
		ln("Launches.");
		Thread t = new Thread(new Listener(), "TestServer.Listener");
		t.start();
	}

	class Listener implements Runnable
	{
		public void run()
		{
			ServerSocket ss;
			try {
				if (1 == 1) {
					List<Socket> list = new ArrayList<>(1000);
					for (int i = 0; i < 5000; i++) {
						System.err.println("Opens socket #" + list.size());
						Socket skt = new Socket("127.0.0.1", 8228);
						OutputStream os = skt.getOutputStream();
						os.write("GET / HTTP/1.1\r\nHost: abc\r\n\r\n".getBytes("ISO-8859-1"));
						list.add(skt);
					}
					System.err.println("COMPLETE!!!");
					return;
				}
				ss = new ServerSocket(8888);
			}
			catch (Exception ex) {
				ln("Got exception while opening server socket: " + ExceptionUtils.getExMsg(ex));
				return;
			}
			for (int i = 0; i < 100; i++) {
				try {
					Socket s = ss.accept();
					new Responder(s);
				}
				catch (Exception ex) {
					ln("Got exception while accepting connection: " + ExceptionUtils.getExMsg(ex));
				}
			}
			try {
				ss.close();
			}
			catch (Exception ex) {
				ln("Got exception while closing server socket: " + ExceptionUtils.getExMsg(ex));
			}
		}
	}

	class Responder implements  Runnable, Logger
	{
		final Socket skt;
		final InputStream is;
		final OutputStream os;
		private Logger connectionLogger;

		Responder(Socket skt) throws IOException
		{
			this.connectionLogger = new PrefixedLogger(logger, Integer.toString(nextConnectionNumber++));
			this.skt = skt;
			this.is = new BufferedInputStream(skt.getInputStream());
			this.os = new BufferedOutputStream(skt.getOutputStream());
			Thread t = new Thread(this, "TestServer.Connection." + skt.getPort());
			t.start();
		}

		public void run()
		{
			try {
				main();
			}
			catch (Exception ex) {
				ln("Got exception while processing request: " + ExceptionUtils.getExMsg(ex));
			}
			finally {
				try {
					skt.close();
				}
				catch (Exception ex) {
					ln("Got exception when closing socket: " + ExceptionUtils.getExMsg(ex));
				}
			}
		}

		private void main() throws Exception
		{
			String req = readHeaderLine();
			ln("Req: " + req);
			while (true) {
				String s = readHeaderLine();
				if (s.length() == 0)
					break;
			}
			String html = "You have three hours. You may use your lecture notes and Feynman. " +
					"I hear Madoff got 150 years. What did they give Paulson? " +
					"On reading these instructions one particularly alert student grabbed his " +
					"exam and raced across campus to Richard Feynman's office. " +
					"He handed the exam to Feynman explaining that the directions clearly indicated " +
					"he was a valid resource. " +
					"Feynman glanced over the instructions and agreed. " +
					"He wrote out the exam in less than half an hour, and got a perfect score!" +
					"";
			byte[] b = html.getBytes("ISO-8859-1");
			sendResponseLine("HTTP/1.1 200 OK");
			sendResponseLine("Content-Type: text/html");
			sendResponseLine("Content-Length: " + (b.length + 1));
			sendResponseLine("Date: Mon, 29 Jun 2009 13:28:56 GMT");
			sendResponseLine("Last-Modified: Mon, 29 Jun 2009 13:28:56 GMT");
			sendResponseLine("Server: VNC Server Enterprise Edition/E4.4.2 (r13117)");
			os.write('\r');
			os.write('\n');
			os.write(b);
			os.flush();
			ln("Sent response.");
			//Thread.sleep(180 * 1000);
		}

		private String readHeaderLine() throws IOException
		{
			StringBuilder sb = new StringBuilder(20);
			while (true) {
				int i = is.read();
				if (i < 0 || i == 10)
					break;
				if (i == 13)
					continue;
				sb.append((char)i);
			}
			return sb.toString();
		}

		private void sendResponseLine(String s) throws IOException
		{
			os.write(s.getBytes("UTF-8"));
			os.write('\r');
			os.write('\n');
		}

		public void ln(@Nullable String s)
		{
			connectionLogger.ln(s);
		}
	}

	public void ln(@Nullable String s)
	{
		logger.ln(s);
	}
}
