/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.admin.misc;

import java.net.InetAddress;
import java.util.regex.Pattern;
import org.glimmerblocker.proxy.admin.AbstractAdminXmlHandler;
import org.glimmerblocker.proxy.proxyRequest.ProxyRequestHandler;
import org.jetbrains.annotations.NotNull;
import org.mozilla.javascript.RhinoException;
import dk.vitality.lib.foundation.exception.ExceptionUtils;

public class AdminSiteProxyValidate extends AbstractAdminXmlHandler
{
	public AdminSiteProxyValidate(@NotNull ProxyRequestHandler req) throws Exception
	{
		super(req);
	}

	protected void processAdminXml() throws Exception
	{
		String js = urlParams.optString("javascript");
		if (js != null) {
			try {
				globals.networkManager.validateProxyJavascript(js);
			}
			catch (Exception ex) {
				String msg = ExceptionUtils.getExMsg(ex);
				msg = rhinoNoiseRE.matcher(msg).replaceAll("");
				if (ex instanceof RhinoException) {
					RhinoException rex = (RhinoException)ex;
					if (rex.lineNumber() > 0)
						msg = "Javascript error in line " + rex.lineNumber() + ": " + msg;
					else
						msg = "Javascript error: " + msg;
				}
				if (globals.debugFlag) {
					ln(msg);
					ln("----- script:\n" + js + (js.endsWith("\n") ? "" : "\n") + "------------");
				}
				rootE.setAttribute("proxy-error", msg);
			}
			return;
		}
		//----------------------------------------------------------------------------------------
		int port = 80;
		String portError = null;
		try {
			String s = urlParams.optString("port");
			if (s != null) {
				port = Integer.parseInt(s);
				if (port < 80 || port > 65535)
					throw new Exception("Must be between 80 and 65535.");
				else if (port == 443)
					throw new Exception("Must NOT be 443 (https).");
			}
		}
		catch (Exception ex) {
			portError = "Port number invalid: " + ExceptionUtils.getExMsg(ex);
		}
		//----------------------------------------------------------------------------------------
		try {
			String h = urlParams.reqString("host");
			InetAddress addr = InetAddress.getByName(h);
			if (globals.debugFlag)
				ln("Resolved site proxy to: " + addr.getHostAddress());
			if (port == globals.httpServer.getListenPort()) {
				if (h.equals("127.0.0.1") || h.equals("localhost") || addr.isAnyLocalAddress())
					throw new Exception("Infinite loop when site-proxy is GlimmerBlocker itself!");
			}
		}
		catch (Exception ex) {
			rootE.setAttribute("proxy-error", "Hostname/ip invalid: " + ExceptionUtils.getExMsg(ex));
			return;
		}
		if (portError != null)
			rootE.setAttribute("proxy-error", portError);
	}

	private static final Pattern rhinoNoiseRE = Pattern.compile(" *\\(script#\\d+\\)");
}