/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.client.cache;

import org.glimmerblocker.proxy.globals.log.Logger;
import org.glimmerblocker.proxy.globals.log.PrefixedLogger;
import org.glimmerblocker.proxy.globals.network.dns.DnsResolver;
import org.glimmerblocker.proxy.http.client.connection.HttpClientConnection;
import org.glimmerblocker.proxy.http.client.connection.KeepAliveException;
import org.glimmerblocker.proxy.proxyRequest.ClientAbortedException;
import org.jetbrains.annotations.NotNull;
import dk.vitality.lib.foundation.ref.FatRef;
import dk.vitality.lib.connectionCache.ConnectionCache;
import dk.vitality.lib.connectionCache.ConnectionInfo;

/**
 * A cache of connections to a single host:port to support keep-alive connections.
 */
public class HostHttpConnectionCache
		extends ConnectionCache<CacheableHttpConnection, HttpClientConnection>
{
	@NotNull final HttpClientCacheKey key;
	@NotNull public final DnsResolver dnsResolver;
	@NotNull private final Logger parentLogger;
	@NotNull private final FatRef<Boolean> debugHttpClient;
	@NotNull private final FatRef<Integer> ipAddressIndex = new FatRef<>(0);
	private int connectionSerial;

	public HostHttpConnectionCache(@NotNull String cacheName,
	                               @NotNull final HttpClientCacheKey key,
	                               @NotNull DnsResolver dnsResolver,
	                               @NotNull Logger parentLogger,
	                               @NotNull FatRef<Boolean> debugHttpClient)
	{
		super(cacheName, Integer.MAX_VALUE);
		this.dnsResolver = dnsResolver;
		this.key = key;
		this.parentLogger = parentLogger;
		this.debugHttpClient = debugHttpClient;
	}

	@NotNull
	protected ConnectionInfo<HttpClientConnection> createNewConnection() throws Exception
	{
		String prefix = key.tcpHost.hostnamePort + " #" + (++connectionSerial);
		Logger connLogger = new PrefixedLogger(parentLogger, prefix);
		HttpClientConnection conn = new HttpClientConnection(dnsResolver, key, key.ssl,
				connLogger, ipAddressIndex, debugHttpClient);
		return new CacheableHttpConnection(conn);
	}

	protected void handleExceptionWhileUsingConnection(@NotNull Throwable ex)
	{
		if (ex instanceof KeepAliveException || ex instanceof ClientAbortedException) {
			return;
		}
		super.handleExceptionWhileUsingConnection(ex);
	}
}
