/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.server;

import java.io.IOException;
import java.io.OutputStream;
import org.jetbrains.annotations.NotNull;

class HttpServerInfiniteOutputStream extends OutputStream
{
	@NotNull final HttpServerRequest req;
	@NotNull final OutputStream os;
	private boolean closed;

	HttpServerInfiniteOutputStream(@NotNull HttpServerRequest req)
	{
		this.req = req;
		this.os = req.conn.os;
	}

	public void write(int b) throws IOException
	{
		if (closed)
			throw new IOException("OutputStream is closed.");
		os.write(b);
	}

	@Override
	public void write(@NotNull byte[] data, int offset, int len) throws IOException
	{
		if (closed)
			throw new IOException("OutputStream is closed");
		os.write(data, offset, len);
	}

	@Override
	public void close() throws IOException
	{
		if (closed)
			return;
		try {
			flush();
		}
		finally {
			closed = true; // must be after flush() to avoid making the flush() call a no-op.
			req.notifyHasSentFullResponseBody();
		}
	}

	@Override
	public void flush() throws IOException
	{
		if (closed)
			return;
		os.flush();
	}
}
