/* Copyright (C) 2008-2012 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>


@interface Xml : NSObject {

}

#pragma mark -------------- Create elements
+ (NSXMLElement*)createElementWithName:(NSString*)name;

#pragma mark -------------- Singletons

// xpaths for these methods *must* be of the form
//  path/to/element
// No attributes, tests,... are allowed.

// if more than 1 child with name: zaps all but the first.
+ (NSXMLElement*)singletonElementForXPath:(NSString*)xpath
								forParent:(NSXMLElement*)parentE;

+ (NSXMLElement*)singletonElementForXPath:(NSString*)xpath
								forParent:(NSXMLElement*)parentE
							 asFirstChild:(BOOL)asFirstChild;

+ (NSXMLElement*)singletonDateElement:(NSDate*)date
							 forXPath:(NSString*)xpath
							fromParent:(NSXMLElement*)parentE
					   updateExisting:(BOOL)updateExisting;

#pragma mark -------------- Attributes
+ (NSString*)getAttribute:(NSXMLElement*)e withName:(NSString*)name;
+ (int)getBoolAttribute:(NSXMLElement*)e withName:(NSString*)name;
+ (int)getIntAttribute:(NSXMLElement*)e withName:(NSString*)name;

+ (void)setAttribute:(NSString*)value withName:(NSString*)name inElement:(NSXMLElement*)e;
+ (void)setBool:(BOOL)value withName:(NSString*)name inElement:(NSXMLElement*)e;
+ (void)setInt:(int)value withName:(NSString*)name inElement:(NSXMLElement*)e;

#pragma mark -------------- Reorder childs
+ (void)reorderChilds:(NSArray*)childs;

#pragma mark -------------------------------- XPath

+ (NSArray *)nodesForXPath:(NSString *)xpath withRoot:(NSXMLElement*)rootE;
+ (NSXMLNode *)nodeForXPath:(NSString *)xpath withRoot:(NSXMLElement*)rootE;
+ (void)remove:(NSString*)xpath withRoot:(NSXMLElement*)rootE;
+ (void)zapChildsAndAttributes:(NSXMLElement*)e;

#pragma mark -------------------------------- simple get/set
// xpaths for these methods *must* be of the form
//  path/to/element/@attrname
+ (NSString*)stringForXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;
+ (void)setString:(NSString*)value forXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;
+ (void)setCData:(NSString*)value forXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;

+ (double)doubleForXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;
+ (void)setDouble:(double)value forXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;

+ (int)intForXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;
+ (void)setInt:(int)value forXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;

+ (BOOL)boolForXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;
+ (void)setBool:(BOOL)value forXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;

+ (NSString*)dateFromMillisecsXPath:(NSString*)xpath withRoot:(NSXMLElement*)rootE;

#pragma mark -------------------------------- regexp
+ (BOOL)regex:(NSString*)regex matchesString:(NSString*)text;
+ (NSString*)getHttpUrlSyntaxError:(NSString*)url;

#pragma mark -------------------------------- unqiue id

+ (int)makeUniqueIdWithRoot:(NSXMLElement*)rootE withAttributeName:(NSString*)attrName;
+ (int)makeUniqueIdWithRoot:(NSXMLElement*)rootE; // attrName = 'id'

#pragma mark -------------------------------- print-to-string
+ (NSData*)documentToData:(NSXMLDocument*)doc;
+ (NSString*)documentToString:(NSXMLDocument*)doc;

+ (NSData*)subtreeToData:(NSXMLElement*)parentE;
+ (NSString*)subtreeToString:(NSXMLElement*)parentE;

@end
